local HavenLoots = LibStub("AceAddon-3.0"):NewAddon("HavenLoots", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0");

local mainTexture = "Interface\\AddOns\\HavenLoots\\Textures\\white.tga";
local MiniMapIconTexture = "Interface\\AddOns\\HavenLoots\\Textures\\MiniMapIcon.tga";
local mainFont = "Fonts\\FRIZQT__.TTF";
local HL_Colors = {{0, 1, 0, 1}, {0, 0, 1, 1}, {1, 1, 0, 1}, {0.57, 0.43, 0.85, 1}, {1, 0, 0, 1}, {0.5, 0.5, 0.5, 1}};
local sendType = "RAID";

--##########################################################################
-- class variables
local itemFrame = {};
local rollWindows = {};
local choiceBoxes = {};
local currentRolls = {};
local closeButton = OhaiyoClasses_BasicFrame.create();
local cpos = 0;
local isShown = false;
local currentVersion = GetAddOnMetadata("HavenLoots", "Version");

for i = 1, 20 do
	itemFrame[i] = OhaiyoClasses_BasicFrame.create();
	choiceBoxes[i] = {};
	rollWindows[i] = {};
	currentRolls[i] = {};
	currentRolls[i].hasRolled = false;
	currentRolls[i].link = nil;
	for x = 1, 40 do
		rollWindows[i][x] = {};
		for y = 1, 6 do
			rollWindows[i][x][y] = OhaiyoClasses_BasicFrame.create();
		end
	end

	for x = 1, 6 do
		choiceBoxes[i][x] = OhaiyoClasses_BasicFrame.create();
	end
end

--##########################################################################
-- check message is from master looter
local function IsMasterLooter(name)
	local _, _, ID = GetLootMethod();
	if ID == nil or UnitName("raid" ..ID) ~= name then
		return false;
	end
	return true;
end

--##########################################################################
-- display tooltip
local function ItemButtonUpdate(self, elapsed)
	if(IsModifiedClick('COMPAREITEMS') or (GetCVarBool('alwaysCompareItems') and not IsEquippedItem(self.itemLink))) then
		GameTooltip_ShowCompareItem();
	else
		ShoppingTooltip1:Hide();
		ShoppingTooltip2:Hide();
	end

	if(IsModifiedClick('DRESSUP')) then
		ShowInspectCursor();
	else
		ResetCursor();
	end
end

local function ItemButtonEnter(self)
	GameTooltip:SetOwner(self, 'ANCHOR_TOPRIGHT');
	GameTooltip:SetHyperlink(self.itemLink);
	self:SetScript('OnUpdate', ItemButtonUpdate);
end

local function ItemButtonLeave(self)
	GameTooltip:Hide();
	self:SetScript('OnUpdate', nil);
end

local function ItemButtonClick(self)
	if IsControlKeyDown() then
		DressUpItemLink(self.itemLink);
	elseif IsShiftKeyDown() then
		ChatEdit_InsertLink(self.itemLink);
	end
end

--##########################################################################
-- add to results table
local function DrawResults()
	for i = 1, 20 do if currentRolls[i] ~= nil then
		for x = 1, 40 do if currentRolls[i][x] ~= nil then
			local name = currentRolls[i][x].name;
			local rollType = currentRolls[i][x].rollType;
			if rollType ~= 6 then
				words = {}
				for word in name:gmatch("([^-]+)") do table.insert(words, word) end
				rollWindows[i][x][rollType]:SetText(words[1] ..": " ..currentRolls[i][x].val - ((5 - currentRolls[i][x].rollType) * 100));
				rollWindows[i][x][rollType]:Show();
				for y = 1, 6 do
					if y ~= rollType then
						rollWindows[i][x][y]:Hide();
					end
				end
			else
				words = {}
				for word in name:gmatch("([^-]+)") do table.insert(words, word) end
				rollWindows[i][x][rollType]:SetText(words[1]);
				rollWindows[i][x][rollType]:Show();
				for y = 1, 5 do
					rollWindows[i][x][y]:Hide();
				end
			end
		end end
	end end
end

local function AddToResults(name, rollType, val, itemLink)
	local itemNumber = nil;
	for i = 1, 20 do
		if currentRolls[i].link ~= nil then
			if currentRolls[i].link == itemLink then
				itemNumber = i;
			end
		end
	end
	if itemNumber ~= nil then
		if rollType ~= 6 then
			for i = 1, table.getn(currentRolls[itemNumber]) do
				if currentRolls[itemNumber][i].val <= val then
					local data = {
						rollType = rollType,
						val = val,
						name = name,
					};
					table.insert(currentRolls[itemNumber], i, data);
					return;
				end
			end
		end
		local data = {
			rollType = rollType,
			val = val,
			name = name,
		};
		table.insert(currentRolls[itemNumber], data);
	end
end

--##########################################################################
-- showing and hiding frames
local function SendLootRoll(self)
	for x = 1, 6 do
		choiceBoxes[self.itemNumber][x]:Hide();
	end
	currentRolls[self.itemNumber].hasRolled = true;
	HavenLoots:SendCommMessage("HavenLoots","roll" .."_" ..currentRolls[self.itemNumber].link .."_" ..self.rollType, sendType);
end

local function CreateLootFrame(link, i)
	itemFrame[i]:Show();
	itemFrame[i]:GetFrame().itemLink = link;
	itemFrame[i]:SetText(link);
	if currentRolls[i].hasRolled ~= true then
		for x = 1, 6 do
			choiceBoxes[i][x]:Show();
		end
	end
end

local function ClearFrames()
	closeButton:Hide();
	for i = 1, 20 do
		itemFrame[i]:Hide();
		for x = 1, 6 do
			choiceBoxes[i][x]:Hide();
		end
		for x = 1, 40 do
			for y = 1, 6 do
				rollWindows[i][x][y]:Hide();
			end
		end
	end
	isShown = false;
end

local function ShowFrames()
	closeButton:Show();
	for i = 1, cpos do
		CreateLootFrame(currentRolls[i].link, i);
	end
	isShown = true;
	DrawResults();
end

local function ToggleAddon()
	if isShown then
		ClearFrames();
	else
		ShowFrames();
	end
end

--##########################################################################
-- frame movement functions
local function MouseDownCloseButton(self, button)
	if button == "RightButton" then
		self:StartMoving();
		self.isMoving = true;
	elseif button == "LeftButton" then
		ClearFrames();
	end
end

local function MouseUpCloseButton(self, button)
	if self.isMoving then
		self:StopMovingOrSizing();
		self.isMoving = false;
		HavenLoots_point, HavenLoots_parent, HavenLoots_relativeTo, HavenLoots_x, HavenLoots_y = self:GetPoint();
	end
end

--##########################################################################
-- set up frames
local function SetUpChoiceBox(box, parent, text, type, inum)
	box:PositionFrames("BOTTOMLEFT", parent, "TOPLEFT", 100, 30, 0, 0);
	local c = HL_Colors[type];
	box:SetTexture(mainTexture, 0, 0, 0, 0.5);
	box:SetBackground(mainTexture, c[1], c[2], c[3], c[4]);
	box:SetFont(mainFont, 15, 1, 1, 1, 1);
	box:SetText(text);
	box:Hide();

	box:GetFrame():EnableMouse(true);
	box:GetFrame().rollType = type;
	box:GetFrame().itemNumber = inum;
	box:GetFrame():RegisterForDrag("LeftButtonDown");
	box:GetFrame():SetScript("OnMouseDown", SendLootRoll);
	return box;
end

local function SetUpLootFrames()
	for i = 1, 20 do
		local parent = closeButton:GetFrame();
		if i ~= 1 then
			parent = itemFrame[i - 1]:GetFrame();
		end
		itemFrame[i]:PositionFrames("TOPLEFT", parent, "TOPRIGHT", 100, 30, 0, 0);
		itemFrame[i]:SetTexture(mainTexture, 0.2, 0.2, 0.2, 0.5);
		itemFrame[i]:SetBackground(mainTexture, 0, 0, 0, 0.7);
		itemFrame[i]:SetFont(mainFont, 10, 1, 1, 1, 1);
		itemFrame[i]:Hide();

		itemFrame[i]:GetFrame():EnableMouse(true);
		itemFrame[i]:GetFrame():RegisterForDrag("LeftButtonDown");
		itemFrame[i]:GetFrame():SetScript("OnMouseDown", ItemButtonClick);
		itemFrame[i]:GetFrame():SetScript('OnEnter', ItemButtonEnter);
		itemFrame[i]:GetFrame():SetScript('OnLeave', ItemButtonLeave);

		choiceBoxes[i][1] = SetUpChoiceBox(choiceBoxes[i][1], itemFrame[i]:GetFrame(), "Main Spec", 1, i);
		choiceBoxes[i][2] = SetUpChoiceBox(choiceBoxes[i][2], choiceBoxes[i][1]:GetFrame(), "Off Spec", 2, i);
		choiceBoxes[i][3] = SetUpChoiceBox(choiceBoxes[i][3], choiceBoxes[i][2]:GetFrame(), "Offline Alt", 3, i);
		choiceBoxes[i][4] = SetUpChoiceBox(choiceBoxes[i][4], choiceBoxes[i][3]:GetFrame(), "Transmog", 4, i);
		choiceBoxes[i][5] = SetUpChoiceBox(choiceBoxes[i][5], choiceBoxes[i][4]:GetFrame(), "Sell", 5, i);
		choiceBoxes[i][6] = SetUpChoiceBox(choiceBoxes[i][6], choiceBoxes[i][5]:GetFrame(), "Pass", 6, i);
	end
end

local function SetUpCloseButton()
	closeButton:PositionFrames(HavenLoots_point, HavenLoots_parent, HavenLoots_relativeTo, 75, 30, HavenLoots_x, HavenLoots_y);
	closeButton:SetTexture(mainTexture, 0.2, 0.2, 0.2, 0.5);
	closeButton:SetBackground(mainTexture, 0, 0, 0, 0.7);
	closeButton:SetFont(mainFont, 15, 1, 1, 1, 1);
	closeButton:SetText("Close");
	closeButton:Hide();

	closeButton:GetFrame():EnableMouse(true);
	closeButton:GetFrame():RegisterForDrag("LeftButtonDown", "RightButtonDown");
	closeButton:GetFrame():SetMovable(true);
	closeButton:GetFrame():SetScript("OnMouseDown", MouseDownCloseButton);
	closeButton:GetFrame():SetScript("OnMouseUp", MouseUpCloseButton);
end

local function SetUpRollWindows()
	for i = 1, 20 do for x = 1, 40 do for y = 1, 6 do
		local parent = itemFrame[i]:GetFrame();
		if x ~= 1 then
			parent = rollWindows[i][x - 1][y]:GetFrame();
		end
		local c = HL_Colors[y];
		rollWindows[i][x][y]:PositionFrames("TOPLEFT", parent, "BOTTOMLEFT", 100, 20, 0, 0);
		rollWindows[i][x][y]:SetTexture(mainTexture, 0, 0, 0, 0.7);
		rollWindows[i][x][y]:SetFont(mainFont, 10, 1, 1, 1, 1);
		rollWindows[i][x][y]:SetBackground(mainTexture, c[1], c[2], c[3], c[4]);
		rollWindows[i][x][y]:Hide();
	end end end
end

--##########################################################################
-- recieve messages
function HavenLoots:OnCommReceived(prefix, message, channel, sender)
	local formattedName = {}
	for name in sender:gmatch("([^-]+)") do table.insert(formattedName, name) end
	if IsMasterLooter(formattedName[1]) then
		words = {}
		for word in message:gmatch("([^_]+)") do table.insert(words, word) end
		if words[1] == "new" then
			for i = 1, table.getn(currentRolls) do
				if currentRolls[i].link == words[2] then
					return;
				end
			end
			cpos = cpos + 1;
			currentRolls[cpos].link = words[2];
			ShowFrames();
		elseif words[1] == "res" then
			words = {}
			for word in message:gmatch("([^_]+)") do table.insert(words, word) end
			AddToResults(words[2], tonumber(words[3]), tonumber(words[4]), words[5]);
			if isShown == true then
				DrawResults();
			end
		elseif words[1] == "clear" then
			for i = 1, 20 do
				currentRolls[i] = {};
				currentRolls[i].hasRolled = false;
				currentRolls[i].link = nil;
			end
			cpos = 0;
			ClearFrames();
		elseif words[1] == "check" then
			HavenLoots:SendCommMessage("HavenLoots", "ver" .."_" ..currentVersion, sendType);
		end
	end
end
HavenLoots:RegisterComm("HavenLoots");

--##########################################################################
-- minimap icon
local function MiniMapButtonToolTip(self)
	GameTooltip:SetPoint("TOPRIGHT", Minimap, "BOTTOMLEFT", -10, -10);
	GameTooltip:SetText("Haven Loots v." ..currentVersion);
end

local HLLDB = LibStub("LibDataBroker-1.1"):NewDataObject("HLButton", {
	type = "data source",
	text = "Haven Loots",
	icon = MiniMapIconTexture,
	OnClick = ToggleAddon,
	OnTooltipShow = MiniMapButtonToolTip,
});
local icon = LibStub("LibDBIcon-1.0");

--##########################################################################
-- start up
function HavenLoots:OnInitialize()
	icon:Register("HLButton", HLLDB, MiniMapButton);

	if MiniMapButton == nil then
		MiniMapButton = {};
		MiniMapButton.hide = false;
	end

	if MiniMapButton.hide then
		icon:Hide("HLButton");
	else
		icon:Show("HLButton");
	end

	if HavenLoots_x == nil then HavenLoots_x = 300 end
	if HavenLoots_y == nil then HavenLoots_y = -300 end
	if HavenLoots_point == nil then HavenLoots_point = "TOP" end
	if HavenLoots_parent == nil then HavenLoots_parent = "UIParent" end
	if HavenLoots_relativeTo == nil then HavenLoots_relativeTo = "TOPLEFT" end

	SetUpCloseButton();
	SetUpRollWindows();
	SetUpLootFrames();
end

--##########################################################################
--  chat commands
HavenLoots:RegisterChatCommand("hl", "OnSlash");
HavenLoots:RegisterChatCommand("havenloots", "OnSlash");
function HavenLoots:OnSlash(msg, editbox)
	if msg == "icon" then
		MiniMapButton.hide = not MiniMapButton.hide;
		if MiniMapButton.hide then
			icon:Hide("HLButton");
		else
			icon:Show("HLButton");
		end
	else
		ToggleAddon();
	end
end